;*************************************************************************
; FILE ..........: P12C509A.asm
; AUTHOR ........: Vitaly Puzrin
; DESCRIPTION ...: Firmware for FMR interface cable
; NOTES .........:
; COPYRIGHT .....: Vitaly Puzrin, 2000    http://www.rcdesign.ru
; HISTORY .......: DATE		COMMENT
;                  ---------------------------------------------------
;                  25.11.2000	First version.
;                  20.12.2001   logic_wizard updated for 12C509A
;*************************************************************************



	list p=12C509A, f=inhx8m

	IFDEF	__12C509A

#include	<p12C509A.inc>
#define		TxBit	GPIO, 0		; 室 
#define		RAM_START	H'07'

	__CONFIG 	_MCLRE_OFF & _CP_OFF & _WDT_ON & _WDT_ON & _IntRC_OSC

	ELSE

#include	<p16c84.inc>
#define		TxBit	PORTB, 2	; 室 
#define		RAM_START	H'0C'

	__CONFIG 	_CP_ON & _PWRTE_ON & _WDT_ON & _XT_OSC

	ENDIF

BAUD_1  	equ	D'15'		; 3 + 5 +3X = CLKOUT/Baud
BAUD_X  	equ	D'12'		; 11 + 5 + 3X = CLKOUT/Baud

;------------------------------------
; ஢ ६
;
	CBLOCK	RAM_START
		PktCnt		; 稪   
		TxData		;   ।
		BitCnt		; 稪   । 
		TimeCnt		; 稪 ⥫쭮 
		DelCnt		; 稪 প

		PktStart	; 砫  
	ENDC

	org	0

;------------------------------------
; ᫥  (HARDWARE  WDT)
; ᥣ 諥  FF
;
Reset:

	IFDEF	__12C509A

	movwf	OSCCAL		; 㥬 ⠪⮢ 

	ENDIF

	clrwdt
	movlw	B'00111010'	; ⠩ ४砥  RA4
				; ४祭   ஭ ᨣ
				; ।⥫ 祭  WDT
				; ਮ WDT - 20 * 4 
	option

	IFDEF	__12C509A

	movlw	0FFh		; ࠨ  -뢮
	movwf	GPIO
	bsf	TxBit
	movlw	B'00011110'	;  GPIO  , ஬ GP0
	tris	GPIO

	ELSE

	movlw	0FFh		; ࠨ  -뢮
	movwf	PORTA
	movwf	PORTB
	bsf	TxBit
	movlw	B'00010000'	;  PORTA ஬ PA4  뢮
	tris	PORTA
	movlw	B'00000000'	;  PORTB  뢮
	tris	PORTB

	ENDIF

	clrwdt			; ᫨ ᨣ ,  㤥 
				; 뢠  ᫠ ⮩ 

	clrf	TMR0		; ⠢ ⠩  ९

	clrf	PktCnt
	goto	send_pkt	;  ⮩ .
				;


;------------------------------------
;------------------------------------
;   ஭.
; (!)  ᠬ ࢮ 祭
;     砫 .
; ⮬  ᨭ஭ 
; ࢮ  㧥.
;
wait_first_front:
	movf	TMR0, W			;    室 ⠩
	btfsc	STATUS, Z
	goto	wait_first_front
;
	nop		; ࠢ ६ ७
	nop		;  ࢮ 
	nop		;
	nop		;
	nop		;
	nop		;

;------------------------------------
; ਭ   4- .
; ᫨ ࠭ ६ 㯠 㧠,
;  ।  FF
;
measure_channel:
;
	clrf	TMR0
	clrwdt

	movlw	D'143'				; প  715 ,
	movwf	DelCnt
DelLp:
	nop
	nop
	decfsz	DelCnt, F
	goto	DelLp

	nop
	nop

;
; 塞 ६  ᫥饣 
;
	movlw	0FFh			; ⠢
	movwf	TimeCnt			; 稪 ६
wait_front_loop:
	movf	TMR0, W			; ⭮ ७ -
	btfss	STATUS, Z		; 7 ᥪ㭤.
	goto	save_measurement
	decfsz	TimeCnt, F		; ᫨ 㧠 ᫨誮 
	goto	wait_front_loop		; (稪 ९) 
					; 稭 ᪠ 
					;  ᠬ 砫
	goto	send_pkt

;------------------------------------
; 뢠 ﭭ 祭
; ⥫쭮 
;
save_measurement:
	clrf	TMR0
	comf	TimeCnt, W		; (!) 㥬 १,
	movwf	INDF			; ⠪  ⠫  ⭮ 浪
	incf	FSR, F
	incf	PktCnt,F
	goto	measure_channel

;------------------------------------
; ⠪, ⠩-.  ३
; 㦭 ࠢ 
;
send_pkt:
	movlw	PktStart
	movwf	FSR

	clrf	TxData
	decf	TxData,F
	call	tx_byte			; ࠢ塞  

	movlw	PktStart
	movwf	FSR

	movf	PktCnt, W
	btfsc	STATUS, Z
	goto	skip_data

pkt_loop:
	movf	INDF, W
	movwf	TxData
	call    tx_byte
	incf	FSR, F
	decfsz	PktCnt, F
	goto	pkt_loop

skip_data:

	clrf	PktCnt
	movlw	PktStart                ; (稭  ࢮ)
	movwf	FSR

	clrf	TMR0			; ⠢ ⠩
					;  ९
	goto	wait_first_front	; 室  
					; ࢮ 

;------------------------------------
;------------------------------------
; । 1   ᫥⥫쭮
; 䥩.  19200 .
; ଠ  - 81N
;
tx_byte:
	movlw	8
	movwf	BitCnt
;
	bcf	TxBit			;  -
	call	Delay1
tx_next_bit:
	bcf	STATUS,C
;
	rrf	TxData,F
;
	btfsc	STATUS,C
	bsf	TxBit
	btfss	STATUS,C
	bcf	TxBit
	call	DelayX
	decfsz	BitCnt,F
	goto	tx_next_bit
	bsf	TxBit		; Send Stop Bit
	call    Delay1
;
	retlw	0
;
; প  । ⮢ 
;
DelayX  movlw   BAUD_X
	goto    save

Delay1  movlw   BAUD_1            ; 52 uS   19200 
	goto    save

save    movwf   DelCnt
redo_1  decfsz  DelCnt,F
	goto    redo_1

	retlw   0

	end
